package com.brackeen.evolution.core;

import com.brackeen.evolution.core.game.GameScene;
import com.brackeen.evolution.core.game.SetupScene;
import playn.core.Color;
import playn.core.Game;
import playn.core.GroupLayer;
import static playn.core.PlayN.*;
import playn.core.Sound;

public class Evolution implements Game {
    
    public static final int WIDTH = 240;
    public static final int HEIGHT = 320;
    // Changing this will probably mess up collisions, velocity
    public static final int SCALE = 2;
    
    public static final int[] COLORS = {
        Color.rgb(0, 0, 0), // Black
        Color.rgb(255, 255, 255), // White
        Color.rgb(255, 28, 36), // Red
        Color.rgb(59, 97, 255), // Blue
        Color.rgb(30, 183, 93), // Green
        Color.rgb(136, 64, 183), // Purple
        Color.rgb(255, 246, 16), // Yellow
        Color.rgb(29, 229, 255), // Cyan
    };
    
    private Scene starsBackgroundScene;
    private Scene currentScene;
    
    public static void playSound(String sound) {
        Sound s = assets().getSound(sound);
        if (s != null) {
            s.setVolume(0.25f); // The volume is too damn high
            s.play();
        }
    }

    @Override
    public void init() {
        
        graphics().setSize(getScreenWidth(), getScreenHeight());
        
        // Load Sounds
        assets().getSound("audio/blip");
        assets().getSound("audio/explode");
        assets().getSound("audio/hit");
        assets().getSound("audio/shoot");
        assets().getSound("audio/lose");
        assets().getSound("audio/win");
        
        GroupLayer root = graphics().rootLayer();
        starsBackgroundScene = new StarsBackgroundScene(this);
        root.add(starsBackgroundScene.rootLayer());
        
        setCurrentScene(new SetupScene(this, null, 1));
    }
    
    public int getScreenWidth() {
        return WIDTH * SCALE;
    }
    
    public int getScreenHeight() {
        return HEIGHT * SCALE;
    }
    
    public void setCurrentScene(Scene scene) {
        if (currentScene != null) {
            graphics().rootLayer().remove(currentScene.rootLayer());
        }
        currentScene = scene;
        if (currentScene != null) {
            graphics().rootLayer().add(currentScene.rootLayer());
        }
    }
    
    @Override
    public void paint(float alpha) {
        // the background automatically paints itself, so no need to do anything here!
    }

    @Override
    public void update(float delta) {
        int iterations = 1;
        if (currentScene != null) {
            iterations = currentScene.getUpdateIterations();
        }
        for (int i = 0; i < iterations; i++) {
            starsBackgroundScene.update(this, delta);
            if (currentScene != null) {
                currentScene.update(this, delta);
            }
        }
    }

    @Override
    public int updateRate() {
        // 60 fps ?
        return 16;
    }
    
    public boolean shouldStarsMove() {
        return currentScene instanceof GameScene;
    }
}
