package com.brackeen.evolution.core;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import playn.core.GroupLayer;
import playn.core.PlayN;

public class Scene implements Iterable<Actor> {
    
    private final GroupLayer rootLayer = PlayN.graphics().createGroupLayer();
    private final List<Actor> actors = new ArrayList<Actor>();
    private final List<Actor> actorsToAdd = new ArrayList<Actor>();
    private boolean isUpdating;

    public GroupLayer rootLayer() {
        return rootLayer;
    }
    
    public void addActor(Actor actor) {
        if (isUpdating) {
            actorsToAdd.add(actor);
        }
        else {
            actors.add(actor);
            rootLayer.add(actor.layer());
            actor.setScene(this);
        }
    }
    
    public void removeActor(Actor actor) {
        actor.setScene(null);
        actors.remove(actor);
        rootLayer.remove(actor.layer());
        actor.notifyRemoved();
    }
    
    @Override
    public Iterator<Actor> iterator() {
        return actors.iterator();
    }
    
    public void update(Evolution evolution, float delta) {
        List<Actor> actorsToRemove = null;
        
        isUpdating = true;
        for (Actor actor : actors) {
            actor.update(evolution, delta);
            if (actor.shouldRemove()) {
                if (actorsToRemove == null) {
                    actorsToRemove = new ArrayList<Actor>();
                }
                actorsToRemove.add(actor);
            }
        }
        isUpdating = false;
        for (Actor actor : actorsToAdd) {
            addActor(actor);
        }
        actorsToAdd.clear();
        
        if (actorsToRemove != null) {
            for (Actor actor : actorsToRemove) {
                removeActor(actor);
            }
        }
    }
    
    public int getUpdateIterations() {
        return 1;
    }
    
}
