package com.brackeen.evolution.core.game;

import com.brackeen.evolution.core.Evolution;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import playn.core.Image;
import pythagoras.f.Point;

public class AlienArmada implements Iterable<Alien> {
    
    private final int screenWidth;
    private final int screenHeight;
    private final int width;
    private final int height;
    private int minSpacing;
    private int maxSpacing;
    private int avgSpacing;
    private float t;
    
    private float offsetX;
    private float offsetY;
    private float spacing;
    
    private List<Alien> aliens = new ArrayList<Alien>();
    
    private Image bulletImage;
    private Image largeBulletImage;
    
    public AlienArmada(int screenWidth, int screenHeight, int y) {
        this.screenWidth = screenWidth;
        this.screenHeight = screenHeight;
        this.width = 8;
        this.height = 5;
        this.minSpacing = screenWidth / (width + 4);
        this.maxSpacing = screenWidth / (width + 2);
        this.avgSpacing = screenWidth / (width + 3);
        
        this.offsetY = y + this.avgSpacing;
        
        this.bulletImage = SpriteCreator.create(Bullet.ALIEN);
        this.largeBulletImage = SpriteCreator.create(Bullet.ALIEN_LARGE);
        reset();
    }
    
    @Override
    public Iterator<Alien> iterator() {
        return aliens.iterator();
    }

    public Image getBulletImage() {
        return bulletImage;
    }

    public Image getLargeBulletImage() {
        return largeBulletImage;
    }
    
    public void reset() {
        t = 0;
        update(null, 0);
    }
    
    public int getWidth() {
        return width;
    }
    
    public boolean isPastFrontLine(Alien alien) {
        float y = alien.layer().transform().ty();
        return y >= getViewLocation(new Point(0, height - 1)).y;
    }
    
    public Point getViewLocation(Point formationPosition) {
        return new Point(
                Math.round(offsetX + screenWidth / 2f - ((width - 1) / 2f - formationPosition.x) * spacing), 
                Math.round(offsetY + minSpacing * formationPosition.y));
    }
    
    public void update(Evolution evolution, float delta) {
        float elapsedTime = delta / 1000.0f;
        t += elapsedTime;
        
        float p = (float)Math.sin(t * Math.PI / 16);
        spacing = minSpacing + p * (maxSpacing - minSpacing);
        
        p = (float)Math.sin(t * Math.PI * 2 / 16);
        offsetX = (p * avgSpacing);
        
    }

    public void addAlien(Alien alien) {
        aliens.add(alien);
        alien.setArmada(this);
        Point p = getViewLocation(alien.getArmadaPosition());
        alien.layer().setTranslation(p.x, p.y - screenHeight/2);
        alien.setState(Alien.State.MOVING_TO_FORMATION);
    }
    
    public void removeAlien(Alien alien) {
        aliens.remove(alien);
    }
    
    public int getNumAliens() {
        return aliens.size();
    }
    
    public int getNumAliensInState(Alien.State state) {
        int count = 0;
        for (Alien alien : aliens) {
            if (alien.getState() == state) {
                count++;
            }
        }
        return count;
    }
    
    public Alien getMostRestedAlien() {
        Alien mostRestedAlien = null;
        for (Alien alien : aliens) {
            if (alien.getState() == Alien.State.IN_FORMATION && alien.isRested()) {
                float p = alien.getDurationInState() / alien.getRestTime();
                if (mostRestedAlien == null || p > mostRestedAlien.getDurationInState() / mostRestedAlien.getRestTime()) {
                    mostRestedAlien = alien;
                }
            }
        }
        return mostRestedAlien;
    }
}
