package com.brackeen.evolution.core.game;

import com.brackeen.evolution.core.Actor;
import com.brackeen.evolution.core.Evolution;
import playn.core.Image;
import playn.core.ImageLayer;
import playn.core.PlayN;

public class Bullet extends Actor {
    
    public static final int[][] SPACESHIP = {
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 0, 1, },
        { 0, 0, 0, 0, 0, 0, 0, 1, },
        { 0, 0, 0, 0, 0, 0, 1, 1, },
        { 0, 0, 0, 0, 0, 0, 1, 2, },
        { 0, 0, 0, 0, 0, 0, 1, 2, },
        { 0, 0, 0, 0, 0, 0, 0, 2, },
        { 0, 0, 0, 0, 0, 0, 0, 2, },
        { 0, 0, 0, 0, 0, 0, 0, 2, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
    };    
    
    public static final int[][] ALIEN = {
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 0, 1, },
        { 0, 0, 0, 0, 0, 0, 0, 1, },
        { 0, 0, 0, 0, 0, 0, 0, 1, },
        { 0, 0, 0, 0, 0, 0, 0, 1, },
        { 0, 0, 0, 0, 0, 0, 2, 1, },
        { 0, 0, 0, 0, 0, 0, 2, 1, },
        { 0, 0, 0, 0, 0, 0, 0, 2, },
        { 0, 0, 0, 0, 0, 0, 0, 2, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
    };
    
    public static final int[][] ALIEN_LARGE = {
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 1, 1, },
        { 0, 0, 0, 0, 0, 0, 0, 1, },
        { 0, 0, 0, 0, 0, 0, 0, 1, },
        { 0, 0, 0, 0, 0, 0, 0, 1, },
        { 0, 0, 0, 0, 0, 0, 0, 1, },
        { 0, 0, 0, 0, 0, 0, 0, 1, },
        { 0, 0, 0, 0, 0, 0, 2, 1, },
        { 0, 0, 0, 0, 0, 0, 2, 1, },
        { 0, 0, 0, 0, 0, 0, 2, 2, },
        { 0, 0, 0, 0, 0, 0, 0, 2, },
        { 0, 0, 0, 0, 0, 0, 0, 2, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
        { 0, 0, 0, 0, 0, 0, 0, 0, },
    };    

    public enum Source {
        ALIEN,
        SPACESHIP
    }
    
    private final Source source;
    private final float speedY;
    private final float damage;
    private float actualY;
    
    public Bullet(Source source, Image image, float x, float y, float speedY, float damage) {
        this.source = source;
        this.speedY = speedY;
        this.damage = damage;
        this.actualY = y;
        
        ImageLayer layer = PlayN.graphics().createImageLayer(image);
        layer.setOrigin(image.width()/2, image.height()/2);
        layer.setTranslation((int)x, (int)y);
        setLayer(layer);
    }

    public Source getSource() {
        return source;
    }

    public float getDamage() {
        return damage;
    }
    
    @Override
    public void update(Evolution evolution, float delta) {
        float elapsedTime = delta / 1000.0f;
        float dy = elapsedTime * speedY;
        actualY += dy;
        
        float x = layer().transform().tx();
        layer().setTranslation((int)x, (int)actualY);
        
        if (actualY - layer().height() >= evolution.getScreenHeight()) {
            setShouldRemove(true);
        }
        else if (actualY + layer().height() <= 0) {
            setShouldRemove(true);
        }
    }
}
