package com.brackeen.evolution.core.game;

import com.brackeen.evolution.core.Evolution;
import com.brackeen.evolution.core.Scene;
import com.brackeen.evolution.core.util.TextElement;
import playn.core.Font;
import playn.core.Image;
import playn.core.ImageLayer;
import playn.core.Layer;
import playn.core.Mouse;
import playn.core.Mouse.ButtonEvent;
import playn.core.PlayN;
import playn.core.ResourceCallback;

public class SetupScene extends Scene {
    
    private Evolution evolution;
    private TextElement reservseLabel;
    private TextElement attackLabel;
    private TextElement instructionLabel;
    private int reserveY;
    private int attackY;
    private int instructionsY;
    
        // Other alien genomes
    //"AAAAAAAAAA4HA4_AAgkkgkkkgkkkgikkgkkkAklkAotkAotFAAoFAAAAAAAAAAAA";
    //"AAAAAACAYACAYbQCYbTSobTSAATSAAbaAAAbAAAaAAYSAbbDAbbDAAbAAAbDAAAA";
    //"AAAAAYbAAbbDYbbbYbTbYbSbAESbAAbbAAYbASbDACbHAAAAAbAAAYAAAAAAAAAA";
    //"AAAAAAYAAAbAAbZbYLLbYbZbAbbdAYrdAYbaADQDAbQCAYbCAADAAYAAADAAAAAA";
    //"AAAAAAAAAAAAAAgkgEgkgcjkgkjkgktlAoltAoksAAgAA_gA4nEA4kkAAkEAAAAA";

    private AlienGenome[] DEFAULT_GENOMES = {
        new AlienGenome(),
        new AlienGenome("AAAAAAAAAA4HA4_AAgkkgkkkgkkkgikkgkkkAklkAotkAotFAAoFAAAAAAAAAAAA"),
        new AlienGenome("AAAAAYbAAbbDYbbbYbTbYbSbAESbAAbbAAYbASbDACbHAAAAAbAAAYAAAAAAAAAA"),
    };
    private AlienGenome[] originalGenomes;
    private AlienGenome[] genomes = new AlienGenome[DEFAULT_GENOMES.length];
    
    public SetupScene(final Evolution evolution, AlienGenome[] prevGenomes, final int level) {
        this.evolution = evolution;
        this.originalGenomes = prevGenomes;
        if (originalGenomes == null) {
            originalGenomes = DEFAULT_GENOMES;
        }
        reserveY = 0;
        attackY = 240;
        instructionsY = 0;//attackY * 2;
        showLabels();
        
//        Image dash = PlayN.assets().getImage("images/dash.png");
//        
//        ImageLayer dash1 = PlayN.graphics().createImageLayer(dash);
//        dash1.setTranslation(0, attackY - 8);
//        rootLayer().add(dash1);
//        
//        ImageLayer dash2 = PlayN.graphics().createImageLayer(dash);
//        dash2.setTranslation(0, instructionsY - 8);
//        rootLayer().add(dash2);
        
        genomes[0] = originalGenomes[0];
        genomes[1] = originalGenomes[1];
        genomes[2] = originalGenomes[2];
        
        createAlien(2, evolution.getScreenWidth() / 2, 200);
        createAlien(1, evolution.getScreenWidth() / 2, 320);
        createAlien(0, evolution.getScreenWidth() / 2, 440);
        
        
        Image goButtonImage = PlayN.assets().getImage("images/button_go.png");
        final ImageLayer goButton = PlayN.graphics().createImageLayer(goButtonImage);
        goButtonImage.addCallback(new ResourceCallback<Image>() {

            @Override
            public void done(Image resource) {
                goButton.setSize(resource.width(), resource.height());
                goButton.setOrigin(resource.width()/2, resource.height()/2);
                goButton.setTranslation(evolution.getScreenWidth()/2, 580);
            }

            @Override
            public void error(Throwable err) {
                //
            }
        
        });
        goButton.setTranslation(evolution.getScreenWidth()/2, 580);
        goButton.addListener(new Mouse.LayerAdapter() {
            @Override
            public void onMouseDown(ButtonEvent event) {
                Evolution.playSound("audio/blip");
                evolution.setCurrentScene(new GameScene(evolution, genomes, level));
            }
        });
        rootLayer().add(goButton);
    }
    
    private void createAlien(int genome, int x, int y) {
        final Alien alien = new Alien(genomes[genome]);
        alien.layer().setTranslation(x, y);
        alien.layer().setScale(3);
        addAlien(alien, genome);
    }
    
    public void addAlien(final Alien alien, final int genome) {
        super.addActor(alien);
        alien.setMouseListener(new Mouse.LayerAdapter() {
                @Override
                public void onMouseDown(ButtonEvent event) {
                    mutateAlien(alien, genome, event.button() != 0);
                }

            });
    }
    
   
    private void mutateAlien(Alien alien, int genome, boolean reset) {
        if (reset) {
            genomes[genome] = originalGenomes[genome];
        }
        else {
            genomes[genome] = alien.getGenome().mutatedCopy();
        }
        Alien newAlien = new Alien(genomes[genome]);
        newAlien.layer().setTranslation(alien.layer().transform().tx(), alien.layer().transform().ty());
        newAlien.layer().setScale(3);
        removeActor(alien);
        addAlien(newAlien, genome);
    }
    
    private void remove(TextElement e) {
        if (e != null) {
            Layer layer = e.layer();
            if (layer.parent() != null) {
                layer.parent().remove(layer);
            }
        }
    }
    
    private boolean isBlank(TextElement e) {
        return (e == null || e.layer().width() == 0);
    }
    
    private void showLabels() {
        remove(reservseLabel);
        remove(attackLabel);
        remove(instructionLabel);
        
        Font font = PlayN.graphics().createFont("prstartk", Font.Style.PLAIN, 8 * Evolution.SCALE);
        Layer.HasSize layer;
        
//        reservseLabel = new TextElement(font, "RESERVE FORCE");
//        reservseLabel.setColor(Evolution.COLORS[1]);
//        layer = reservseLabel.layer();
//        //layer.setOrigin(layer.width()/2, 0);
//        reservseLabel.layer().setTranslation(10, reserveY + 10);
//        rootLayer().add(reservseLabel.layer());
//        
//        attackLabel = new TextElement(font, "ATTACK FORCE");
//        attackLabel.setColor(Evolution.COLORS[1]);
//        layer = attackLabel.layer();
//        //layer.setOrigin(layer.width()/2, 0);
//        attackLabel.layer().setTranslation(10, attackY + 10);
//        rootLayer().add(attackLabel.layer());
        
        instructionLabel = new TextElement(font, 
                "CLICK UNIT TO MUTATE\n" +
                "\n" +
                "RIGHT-CLICK TO RESET");
        instructionLabel.setColor(Evolution.COLORS[7]);
        layer = instructionLabel.layer();
        //layer.setOrigin(layer.width()/2, 0);
        instructionLabel.layer().setTranslation(10, instructionsY + 10);
        rootLayer().add(instructionLabel.layer());
    }
    
    @Override
    public void update(Evolution evolution, float delta) {
        if (isBlank(reservseLabel) || isBlank(attackLabel) || isBlank(instructionLabel)) {
            // Font loading problems?
            showLabels();
        }
    }

}
