package com.brackeen.evolution.core.game;

import com.brackeen.evolution.core.Actor;
import com.brackeen.evolution.core.Evolution;
import playn.core.Canvas;
import playn.core.CanvasImage;
import playn.core.Graphics;
import playn.core.ImageLayer;
import playn.core.Layer;
import playn.core.PlayN;

public class Star extends Actor {
    
    private final float depth;
    private final float frequency;
    private float t;

    public Star(float x, float y, int color, float depth, float frequency) {
        this.depth = depth;
        this.frequency = frequency;
        this.t = (float)Math.random();
        
        Graphics g = PlayN.graphics();
        int s = Evolution.SCALE;
        CanvasImage image = g.createImage(s, s);
        Canvas c = image.canvas();
        c.setFillColor(color);
        c.fillRect(0, 0, s, s);
        ImageLayer layer = g.createImageLayer(image);
        layer.setTranslation((int)x, (int)y);
        layer.setAlpha(currentAlpha());
        setLayer(layer);
    }
    
    public float currentAlpha() {
        float p = t * frequency;
        return 0.25f + 0.75f * depth * Math.abs((float)Math.sin(p * (float)Math.PI));
    }
    
    @Override
    public void update(Evolution evolution, float delta) {
        float elapsedTime = delta / 1000.0f;
        t += elapsedTime;
        
        Layer layer = layer();
        layer.setAlpha(currentAlpha());
        if (evolution.shouldStarsMove()) {
           float x = layer.transform().tx();
           float y = layer.transform().ty();
           int dy = Math.max(1, (int)(depth * 6));
           y = (y + dy) % evolution.getScreenHeight();
           layer.setTranslation(x, y);
        }
    }
}
