package com.brackeen.evolution.core.util;

import playn.core.CanvasImage;
import playn.core.Font;
import playn.core.Graphics;
import playn.core.ImageLayer;
import playn.core.Layer;
import playn.core.PlayN;
import playn.core.TextFormat;
import playn.core.TextLayout;

public class TextElement {

    private final ImageLayer layer;
    private TextFormat format;
    private CanvasImage image;
    private int color = 0xff000000;
    private String text;
    private boolean needsRedraw = false;
    
    public TextElement() {
        this(new TextFormat(), "");
    }
    
    public TextElement(String text) {
        this(new TextFormat(), text);
    }
    
    public TextElement(Font font) {
        this(new TextFormat().withFont(font), "");
    }
    
    public TextElement(Font font, String text) {
        this(new TextFormat().withFont(font), text);
    }

    public TextElement(TextFormat format, String text) {
        this.format = format;
        this.layer = PlayN.graphics().createImageLayer();
        this.text = text;
        needsRedraw = true;
    }
    
    public int getColor() {
        return color;
    }
    
    public void setColor(int color) {
        this.color = color;
        needsRedraw = true;
    }

    public TextFormat getFormat() {
        return format;
    }

    public void setFormat(TextFormat format) {
        this.format = format;
        needsRedraw = true;
    }
    
    public Layer.HasSize layer() {
        if (needsRedraw) {
            redraw();
            needsRedraw = false;
        }
        return layer;
    }

    public void setText(String text) {
        this.text = text;
        needsRedraw = true;
    }
    
    private void redraw() {
        needsRedraw = false;
        if (text == null || text.length() == 0) {
            if (image != null) {
                image.canvas().clear();
            }
            return;
        }
        Graphics g = PlayN.graphics();
        TextLayout layout = g.layoutText(text, format);
        if (image == null || layout.width() > image.width() || layout.height() > image.height()) {
            image = g.createImage((int)(layout.width() + 0.5f), (int)(layout.height() + 0.5f));
            layer.setImage(image);
        }
        else {
            image.canvas().clear();
        }
        image.canvas().setFillColor(color);
        image.canvas().fillText(layout, 0, 0);
    }
}
